package de.fhg.iais.kd.gridr.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.io.*;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;

public class CondorRWebserviceUtil
{


	/**
	 * 
	 * @param rCodeToExecute 
	 * 							the code which will be executed in the grid
	 * 
	 * @param functionLib 
	 * 							the functions and variables saved in R with save(... asci=TRUE)
	 * 
	 * @param inputdir
	 * 							File(s) which will be copied to the execution host, ie. a gsiftp URI
	 * @param username
	 * 							myproxy username
	 * @param pwd
	 * 							myproxy password
	 * @return result 
	 * 							the result saved in R with save(asci=TUE)
	 * 
	 */
	public DataHandler executeGridR(String rCode, String localLibFile) {
		
		String rCodeToExecute=null;
		try {
			rCodeToExecute = convertFileToString(rCode);
		} catch (FileNotFoundException e1) {
			System.err.println("Error, cannot open remote File:"+ rCode);
			e1.printStackTrace();
			return(new DataHandler("Error, cannot open remote File:"+ rCode, "text/plain"));
		} catch (IOException e1) {
			System.err.println("Error, cannot open remote File:"+ rCode);
				e1.printStackTrace();
				return(new DataHandler("Error, cannot open remote File:"+ rCode, "text/plain"));
		}
		//delete File rCode
		File f = new File(rCode);
		f.deleteOnExit();
		
		String localTmpFolder = "/tmp/gridRtmp" + System.currentTimeMillis() + "/";
		File localTmp = new File(localTmpFolder);
		if(localTmp.mkdir()==false)
			{
			return (new DataHandler("cannot create local Tmp Folder", "text/plain"));
			}
		//write rCodeToExecute
		
		//replace in rCodeToExecute rCodeLib with localLibFile
		String libFile= new File(localLibFile).getName();
		rCodeToExecute=rCodeToExecute.replace("rCodeLib", libFile);
		
		createFileFromString(localTmpFolder+"remRScript",rCodeToExecute);
		
		//make condor File
		String condor="Executable     = /usr/bin/R \nUniverse       = vanilla \nshould_transfer_files = YES \nwhen_to_transfer_output = ON_EXIT \narguments      = \"CMD BATCH remRScript\"\nError          = err \nLog             = log\ntransfer_input_files =remRScript,"+localLibFile+" \ntransfer_files = ALWAYS \nQueue";
		createFileFromString(localTmpFolder+"condorScript",condor);
		
		try {//condor_submit condorScript
			ProcessBuilder pb = new ProcessBuilder( "condor_submit", "condorScript" );
			pb.directory(localTmp);
			Process p = pb.start();
						
			BufferedReader br = new BufferedReader(new InputStreamReader(p.getErrorStream()));
			String err=br.readLine();
			if(err!=null)
			{
				String retErr=err;
				while((err=br.readLine())!=null) {
					retErr+=err; }
				//delete files
				delDirectory(localTmp);
				return (new DataHandler("Condor Error: "+retErr, "text/plain"));
			}
		} catch (IOException e) {
			e.printStackTrace();
			//delete files
			delDirectory(localTmp);
			return (new DataHandler("Error: "+e, "text/plain"));
		}
		
		File yOut=new File(localTmpFolder + "y.dat");
		System.out.println("waiting for file:"+ localTmpFolder + "y.dat" );
		while(!yOut.exists())
		{
			try {
				Thread.sleep(5000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				//delete files
				delDirectory(localTmp);
				return (new DataHandler("Error: "+e, "text/plain"));
			}
			
		}
		//read y.dat
		System.out.println("ready");
		try {
			FileDataSource res =new FileDataSource(localTmpFolder + "y.dat");
			//delete files
			new File(localLibFile).delete();
			delDirectory(localTmp);
			return (new DataHandler(res));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			//delete files
			new File(localLibFile).delete();
			delDirectory(localTmp);
			return (new DataHandler("Error: "+e, "text/plain"));
		}
		
	}
	private static File createFileFromString(String name, String rCodeToExecute) {
		// create file from R code
		
		Writer fw = null;
		try {
			fw = new FileWriter(name);
			fw.write(rCodeToExecute);

		} catch (IOException e) {
			System.err.println("Cannot create local File:"+ name);

		} finally {
			if (fw != null)
				try {
					fw.close();
				} catch (IOException e) {
				}
		}
		return new File(name);
	}
	private static String convertFileToString(String xmlFile) throws IOException,
	FileNotFoundException {
		FileReader fileReader = new FileReader(xmlFile);
		BufferedReader bufferedReader = new BufferedReader(fileReader);
		StringWriter stringWriter = new StringWriter();
		String line = null;
		String fileContent;
		while ((line = bufferedReader.readLine()) != null) {
			stringWriter.write(line);
			stringWriter.write("\n");
		}
		fileContent = stringWriter.toString();
		return fileContent;
	}
	
	private void delDirectory(File f) {
		
		if(f.exists() && f.isDirectory()) {
			File[] files =f.listFiles();
			for(int i=0; i<files.length; i++)
				files[i].deleteOnExit();
			f.deleteOnExit();
			}
			else
				System.err.println("directory "+ f+ " doesnt exists");
	}
}
